#--------------------------------------------------------------
# Makefile for jPVM interface
#--------------------------------------------------------------
# 
# Author:	David A. Thurman
#		dave@chmsr.isye.gatech.edu
#
#		Center for Human-Machine Systems Research
#		School of Industrial and Systems Engineering
#		Georgia Institute of Technology
#		Atlanta, Georgia 30332-0205
#
#--------------------------------------------------------------
# Installing:
#
# Hopefully, all you have to change is the package locations,
# and the PLATFORM specifier, but I've only compiled this under
# Solaris and Irix. 
#
# Change PLATFORM to the Java representation of your platform OS.
# Use 'solaris' for Sun's running Solaris 2.x.  Use 'irix' for
# SGIs running Irix 5.x (don't know about 6.x yet).  Other than that,
# use whatever Java identifies your system as.  Hint:  Look in
# $JAVA_HOME/include for a OS specific directory.
#
# Platform specific notes:
#
# AIX 
#	add -DIMA_RS6K to CLFAGS
#
#--------------------------------------------------------------
# If someone wants to convert this to AIMK format, I'll put you in the
# acknowledgements.  Gee, your name in print!
#
#--------------------------------------------------------------
#
# Requirements:
#
#	Java		http://www.javasoft.com
#
#	PVM		http://www.epm.ornl.gov/pvm
#
#	Gnu make	ftp://prep.ai.mit.edu
#
#	gcc		ftp://prep.ai.mit.edu
#
# Environment variables
#
# 	PVM_ROOT	- set to top leve directory of PVM
#
#	JAVA_HOME	- set to top level directory of Java
#
#--------------------------------------------------------------
# Build platform - see installation notes above

JAVA_HOME	=	C:\Progra~1\java\jdk1.1.5
CC_HOME		=	C:\Progra~1\Devstudio\VC
#PVM_ROOT	=	(from our enviroment) C:\Progra~1\pvm3.4
PLATFORM	=	WIN32

# PVM source code locations

PVM_SRC_DIR 	= 	$(PVM_ROOT)/src
PVM_XDR_DIR 	= 	$(PVM_ROOT)/xdr
PVM_GSRC_DIR	=	$(PVM_ROOT)/pvmgs

# PVM source code for shared library (group library too)

PVM_SRC 	= 	imalloc.c tev.c lpvmcat.c lpvmgen.c \
					lpvmpack.c lpvmglob.c pmsg.c pvmalloc.c \
					pvmcruft.c pvmerr.c pvmfrag.c waitc.c \
					pvmregex.c regex.c global.c \
					lpvm.c pvmdabuf.c  
PVM_WSRC	=	pvmwin.c xdr.c xdr_floa.c xdr_mem.c	      
PVM_GSRC	=	pvmgsu_core.c pvmgsu_aux.c pvmgs_func.c			      
PVM_OBJ 	=	$(PVM_SRC:.c=.obj) $(PVM_WSRC:.c=.obj) $(PVM_GSRC:.c=.obj)

# jPVM interface source code for shared library	

INT_SRC 	= 	pvmInterface.c 

INT_OBJ		= 	$(INT_SRC:.c=.obj)

# commands
	
CC		=	$(CC_HOME)\bin\cl.exe

# flags - you may have to change -fpic and -shared for your compiler
#	Add -DIMA_RS6K to CLFAGS for AIX machines

PVMWINFLAGS	=	-DHASSTDLIB -DWIN32 -DSYSVBFUNC -DSYSVSTR \
				-DHASERRORVARS -DNOWAIT3 -DNOGETDTBLSIZ -DNOUNIXDOM \
				-DNOTMPNAM -DMAXHOSTNAMELEN=64 -DMAXPATHLEN=64 \
				-DNEEDSFFS
JAVA_INCLUDES 	= 	-I$(JAVA_HOME)/include \
					-I$(JAVA_HOME)/include/$(PLATFORM)
PVM_INCLUDES 	= 	-I$(PVM_ROOT)/include \
					-I$(PVM_ROOT)/src
CC_INCLUDES		=	-I$(CC_HOME)\include
JFLAGS		=	-deprecation
CFLAGS		=	-W2 $(PVMWINFLAGS) -MD /nologo /GX /O2 /D "NDEBUG" \
				/D "_WINDOWS" $(JAVA_INCLUDES) $(PVM_INCLUDES) $(CC_INCLUDES)
LDFLAGS		=	/MACHINE:IX86 /nologo /subsystem:windows /dll
LD_LIBS		=	$(CC_HOME)\lib\wsock32.lib $(CC_HOME)\lib\advapi32.lib
#
# distribution ID
#
DISTRIBUTION 	= 	jPVM-v1.1.3b
#
# rules
# 
all : jPVM.dll

doc :
	javadoc -author -version -d docs jPVM.java

clean :
	$(RM) *.class *.obj *.dll *.lib jPVM.c *.h

#
# dependency rules for JAVA files
#
jPVM.class : jPVM.java
	$(JAVA_HOME)\bin\javac $(JFLAGS) jPVM.java

jPVM.h : jPVM.class
	$(JAVA_HOME)\bin\javah -jni jPVM

pvmInterface.obj : jPVM.h 


# dependency rules for PVM shared library files

imalloc.obj : $(PVM_SRC_DIR)\imalloc.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\imalloc.c

lpvm.obj : $(PVM_SRC_DIR)\lpvm.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\lpvm.c

lpvmcat.obj : $(PVM_SRC_DIR)\lpvmcat.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\lpvmcat.c

lpvmgen.obj : $(PVM_SRC_DIR)\lpvmgen.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\lpvmgen.c

lpvmpack.obj : $(PVM_SRC_DIR)\lpvmpack.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\lpvmpack.c

pvmalloc.obj : $(PVM_SRC_DIR)\pvmalloc.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pvmalloc.c

pmsg.obj : $(PVM_SRC_DIR)\pmsg.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pmsg.c

waitc.obj : $(PVM_SRC_DIR)\waitc.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\waitc.c

global.obj : $(PVM_SRC_DIR)\global.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\global.c

lpvmglob.obj : $(PVM_SRC_DIR)\lpvmglob.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\lpvmglob.c

pvmwin.obj : $(PVM_SRC_DIR)\pvmwin.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pvmwin.c

pvmerr.obj : $(PVM_SRC_DIR)\pvmerr.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pvmerr.c

tev.obj : $(PVM_SRC_DIR)\tev.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\tev.c

pvmcruft.obj : $(PVM_SRC_DIR)\pvmcruft.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pvmcruft.c

pvmdabuf.obj : $(PVM_SRC_DIR)\pvmdabuf.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pvmdabuf.c

pvmfrag.obj : $(PVM_SRC_DIR)\pvmfrag.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\pvmfrag.c

pvmregex.obj: $(PVM_SRC_DIR)\regex\pvmregex.c
	$(CC) $(CFLAGS) -c $(PVM_SRC_DIR)\regex\pvmregex.c

regex.obj: $(PVM_SRC_DIR)\regex\regex.c
	$(CC) $(CFLAGS) -DREGEX_MALLOC -DHAVE_STRING_H \
		-c $(PVM_SRC_DIR)\regex\regex.c

xdr.obj : $(PVM_XDR_DIR)\xdr.c
	$(CC) $(CFLAGS) -c $(PVM_XDR_DIR)\xdr.c

xdr_floa.obj : $(PVM_XDR_DIR)\xdr_floa.c
	$(CC) $(CFLAGS) -c $(PVM_XDR_DIR)\xdr_floa.c

xdr_mem.obj : $(PVM_XDR_DIR)\xdr_mem.c
	$(CC) $(CFLAGS) -c $(PVM_XDR_DIR)\xdr_mem.c

pvmgsu_core.obj : $(PVM_GSRC_DIR)\pvmgsu_core.c
	$(CC) $(CFLAGS) -c $(PVM_GSRC_DIR)\pvmgsu_core.c

pvmgsu_aux.obj : $(PVM_GSRC_DIR)\pvmgsu_aux.c
	$(CC) $(CFLAGS) -c $(PVM_GSRC_DIR)\pvmgsu_aux.c

pvmgs_func.obj : $(PVM_GSRC_DIR)\pvmgs_func.c
	$(CC) $(CFLAGS) -c $(PVM_GSRC_DIR)\pvmgs_func.c

# rules for shared (dynamic link) library

jPVM.dll : $(PVM_OBJ) $(INT_OBJ)
	$(CC_HOME)\bin\link $(LDFLAGS) $(LD_LIBS) $(PVM_OBJ) $(INT_OBJ) -out:jPVM.dll

#install :
#	copy jPVM.dll to the \winnt\system32 directory
