// $Id: DataClient.java,v 1.2 1996/12/16 21:10:03 dave Exp $
// --------------------------------------------------------------------
// JavaPVM - an interface to the Parallel Virtual Machine for Java
// --------------------------------------------------------------------
//
// FILE:	$RCSfile: DataClient.java,v $
//
// AUTHOR:	David A. Thurman 
//		Center for Human-Machine Systems Research
//		Georgia Institute of Technology
//		Atlanta, Georgia
//
//		dave@chmsr.gtech.edu
//
// PURPOSE:	Subclass of JavaPVM to provide necessary functionality
//		for interprocess communications via a separate thread.
//
// --------------------------------------------------------------------
//
// $Log: DataClient.java,v $
// Revision 1.2  1996/12/16 21:10:03  dave
// Minor changes for new version (version number & date).
//
// Revision 1.1  1996/06/04 15:07:52  dave
// Initial revision
//
//
// --------------------------------------------------------------------
// Permission to use and modify this code is hereby granted free of 
// charge, so long as the above comment/acknowledgements block remains
// unchanged.
// --------------------------------------------------------------------

import java.awt.Frame;
import java.awt.Button;
import java.awt.Label;
import java.awt.TextArea;
import java.awt.TextArea;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Color;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Graphics;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * DataClient provides the GUI for the example PVM test program.
 */

class DataClient extends Frame implements iConst
{
  private ExamplePVM	_pvmConn;
  private Button	_sendB;
  private Button	_doneB;
  private Button	_bnchB;
  private TextArea 	_sourceTA;
  private TextArea 	_sinkTA;

	// ---------------------------------------------------
	// CONSTRUCTOR
	// ---------------------------------------------------

  DataClient(String myTitle, ExamplePVM pvmConn)
  {
    super(myTitle);

    _pvmConn = pvmConn;

    init();
  }

	/**
	 * init - Routine to construct the GUI display.
	 */

  private void init()
  {
	// initialize layout information

    Font h18b = new Font("Helvetica", Font.BOLD, 18);
    Font h14p = new Font("Helvetica", Font.PLAIN, 14);
    Font t12i = new Font("Times", Font.ITALIC, 12);

    setBackground(Color.black);
    setForeground(Color.white);
    setFont(h14p);

    GridBagLayout grid = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(grid);
    gbc.fill = GridBagConstraints.BOTH;

	// add a label

    gbc.gridwidth = GridBagConstraints.RELATIVE;
    gbc.gridheight = 2;
    Label dataL = new Label(" JavaPVM Tester ", Label.CENTER);
    dataL.setForeground(_JavaPvmGreen);
    dataL.setFont(h18b);
    grid.setConstraints(dataL, gbc);
    add(dataL);

        // add a version label
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    Label versionL = new Label("JavaPVM-v1.1.3b", Label.CENTER);
    versionL.setFont(t12i);
    versionL.setForeground(_JavaPvmYellow);
    grid.setConstraints(versionL, gbc);
    add(versionL);

        // add a label

    gbc.gridwidth = GridBagConstraints.RELATIVE;
    Label ackL = new Label("Data Client", Label.CENTER);
    ackL.setForeground(_JavaPvmPink);
    ackL.setFont(h14p);
    grid.setConstraints(ackL, gbc);
    add(ackL);

        // add a date label

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    Label dateL = new Label("January 1998", Label.CENTER);
    dateL.setFont(t12i);
    dateL.setForeground(_JavaPvmPink);
    grid.setConstraints(dateL, gbc);
    add(dateL);

	// add label for source text

    Label sourceL = new Label("Edit Text String Window");
    grid.setConstraints(sourceL, gbc);
    sourceL.setFont(t12i);
    add(sourceL);

      // add a source text field - whole line
  
    _sourceTA = new TextArea(6, 20);
    _sourceTA.setEditable(true);
    _sourceTA.setForeground(_JavaPvmGreen);
    grid.setConstraints(_sourceTA, gbc);
    add(_sourceTA);

      // add a button for sending data
   
    _sendB = new Button("Send Text String");
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.setActionCommand("Send String");
    _sendB.addActionListener(new SendActionHandler());

    _sendB = new Button("Send Bytes");
    gbc.gridwidth = GridBagConstraints.RELATIVE;
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.addActionListener(new SendActionHandler());

    _sendB = new Button("Send Doubles");
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.addActionListener(new SendActionHandler());

    _sendB = new Button("Send Floats");
    gbc.gridwidth = GridBagConstraints.RELATIVE;
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.addActionListener(new SendActionHandler());

    _sendB = new Button("Send Integers");
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.addActionListener(new SendActionHandler());

    _sendB = new Button("Send Long");
    gbc.gridwidth = GridBagConstraints.RELATIVE;
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.addActionListener(new SendActionHandler());

    _sendB = new Button("Send Short");
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    grid.setConstraints(_sendB, gbc);
    _sendB.setForeground(_JavaPvmGreen);
    _sendB.setFont(h18b);
    add(_sendB);
    _sendB.addActionListener(new SendActionHandler());

	// add label for sink text

    Label sinkL = new Label("Receiving window");
    sinkL.setFont(t12i);
    grid.setConstraints(sinkL, gbc);
    add(sinkL);

      // add a sink text field

    _sinkTA = new TextArea(6, 20);
    _sinkTA.setEditable(false);
    _sinkTA.setForeground(_JavaPvmYellow);
    grid.setConstraints(_sinkTA, gbc);
    add(_sinkTA);

	// add a close button
    
    _doneB = new Button("Close");
    _doneB.setFont(h18b);
    _doneB.setForeground(_JavaPvmYellow);
    grid.setConstraints(_doneB, gbc);
    add(_doneB);
    _doneB.addActionListener(new DoneActionHandler());

      // add a button for benchmarking
  
    _bnchB = new Button("Benchmark");
    grid.setConstraints(_bnchB, gbc);
    _bnchB.setFont(h18b);
    add(_bnchB);
    _bnchB.addActionListener(new SendActionHandler());

    pack();

    setVisible(true);
  }

	/**
	 * Takes input string and places it in the
	 * output TextArea
	 */

  public void displayMsg(String msg)
  {
    _sinkTA.append(msg + "\n");
  }



/**
 * INNER CLASS - SendActionHandler
 */

  class SendActionHandler implements ActionListener
  {
    public void actionPerformed(ActionEvent ae)
    {
      String Action, Contents = "";

      Action = ae.getActionCommand();


      if (_pvmConn.isBenchmarking())
      {
          System.out.println("**** Wait for benchmarking to complete before sending messages.");
      }
      else if (Action == "Benchmark")
      {
          _pvmConn.Benchmark();
      }
      else
      {
          if (Action == "Send String")
          {
              Contents = _sourceTA.getText();
              _sourceTA.setText("");
          }

          _pvmConn.sendMsg(Action, Contents);
      }
    }
  }

/**
 * INNER CLASS - DoneActionHandler
 */

  class DoneActionHandler implements ActionListener
  {
    public void actionPerformed(ActionEvent ae)
    {
      System.exit(0);
    }
  }

}	// end class PvmTest
