// $Id: ExamplePVM.java,v 1.1 1996/06/04 15:07:52 dave Exp $
// --------------------------------------------------------------------
// JavaPVM - an interface to the Parallel Virtual Machine for Java
// --------------------------------------------------------------------
//
// FILE:	$RCSfile: ExamplePVM.java,v $
//
//
// --------------------------------------------------------------------
//
// $Log: ExamplePVM.java,v $
// Revision 1.1  1996/06/04 15:07:52  dave
// Initial revision
//
//
// --------------------------------------------------------------------
// Permission to use and modify this code is hereby granted free of 
// charge, so long as the above comment/acknowledgements block remains
// unchanged.
// --------------------------------------------------------------------

	/*
	 * Subclass of JavaPVM to provide necessary functionality
	 * for interprocess communications via a separate thread.
	 *
	 * @author:  Dave Thurman<BR>
	 * Center for Human-Machine Systems Research
	 * School of Industrial and Systems Engineering
	 * Georgia Institute of Technology
	 */

public class ExamplePVM implements Runnable, iConst
{
	// private variables

  private Thread	_thread;
  private int		_tid = -1;
  private int		_otherTid = -1;
  private PvmTest	_pvmTest;
  private String	pvmGroup = "ExampleGroup";
  private boolean	_isBenchmarking = false;

	/*
	 * Calls PVM class construtor, caches PVM id of this
	 * process and creates the port watcher thread.
	 */

  ExamplePVM(PvmTest pt)
  {
    _pvmTest = pt;

    _tid = JavaPVM.mytid();		// cache result of native method

    _thread = new Thread(this);	// create thread for this PVM connection
  }

	/**
	 * Called when we're ready to start watching for input.
	 * This starts the Thread running (i.e., forces a call
	 * to the run() method by the Thread.
	 */
  
  public void start()
  {
    _thread.start();
  }

	/**
	 * Returns cached TID w/o resorting to native call.
	 */

  public int getTID()
  {
    return _tid;
  }

	
	/**
	 * Every Runnable subclass needs a run() method.  It's
	 * what it does with it's thread of execution (sort of 
	 * like a main() function in a program).  In this class,
	 * run() watches the PVM connection for input from other
	 * PVM clients.
	 *
	 * It can be customized to handle special msgtags identifying
	 * messages from particular clients.
	 */

  public void run()
  {
    int 	stat;
    int		bufId;
    int		bufInfo;
    int		otherTID[] = new int[1];

	// first get my PVM id and cache it

    _tid = JavaPVM.mytid();	

    System.out.println("ExamplePVM:  My PVM id = " + _tid);

     	// then join the ExamplePVM group

    if ( (stat = JavaPVM.joingroup(pvmGroup)) < 0)
    {
      System.err.println("Couldn't join ExamplePVM group: stat = " + stat);
      abort();
    }

	// then wait for at least one more copy of me to join group

    System.out.println("ExamplePVM:  Waiting for other processes.");

    if ( (stat = JavaPVM.barrier(pvmGroup, 2)) < 0)	// oops
    {
      System.err.println("barrier failed: stat = " + stat + " Exiting...");
      abort();
    }

    _pvmTest.enableGUI();
    
    System.out.println("ExamplePVM:  Got one.");

	// then send my TID out to other process (copy of me)

    int sendTID[] = new int[1];
    sendTID[0] = _tid;

    JavaPVM.initsend(JavaPVM.PvmDataRaw);
    JavaPVM.pkint(sendTID, 1, 1);
    JavaPVM.bcast(pvmGroup, _TidMsg);

	// blocking read to get other process' TID

    if ( (bufId = JavaPVM.recv(-1, _TidMsg)) < 0)
    {
      System.err.println("ExamplePVM:  Error getting other PVM id:  bufId = " 
	+ bufId);
      abort();
    }
    else
    {
      JavaPVM.upkint(otherTID, 1, 1);
      _otherTid = otherTID[0];
      System.out.println("ExamplePVM:  Got other PVM id = " + _otherTid);
    }

	// infinite loop waiting for and handling messages

    System.out.println("ExamplePVM:  Starting up PVM watcher...");

    while (true)
    {
		// blocking receiving for any message

      bufId = JavaPVM.nrecv(-1, -1);

		// ignore errors

      if (bufId < 0)
	continue;

      if (bufId == 0)
      {
	try { Thread.sleep(100); } catch(InterruptedException ie) {;}
	continue;
      }

		// get bufinfo to determine message type

      Integer	msgtag = new Integer(0);
      Integer	bytes = new Integer(0);
      Integer	tid = new Integer(0);

      bufInfo = JavaPVM.bufinfo(bufId, bytes, msgtag, tid);

		// switch on msgtag value

      switch(msgtag.intValue())
      {
	case _TaskExit:
	{
	  abort();
	  break;
	}

	case _StringMsg:
	{
	  char	strMsg[] = new char[1000];

	  if ( (stat = JavaPVM.upkstr(strMsg)) == 0 )	// 0 = success
	  {
	    System.out.println("ExamplePVM:  Got message: " + strMsg);
	    _pvmTest.handleInput(new String(strMsg));
	    _pvmTest.handleInput(new String("\n"));
	  }
	  break;
	}

        case _IntMsg:
        {
            int i;
            int message[] = new int[10];
            String output = new String("Received ints: ");

            if ( (stat = JavaPVM.upkint(message, 10, 1)) == 0)
            {
                System.out.println("ExamplePVM:  Got Integer");
                for (i=0; i<10; i++)
                    output = output + " -" + message[i];
                _pvmTest.handleInput( output );
            }
            else
            {
                System.err.println("ExamplePVM: ERROR***- pvm_upkint returned " + stat);
            }
	  break;

        }

        case _ByteMsg:
        {
            int i;
            byte message[] = new byte[10];
            String output = new String("Received bytes: ");

            System.out.println("ExamplePVM:  Got bytes");

            if ( (stat = JavaPVM.upkbyte(message, 10, 1)) == 0)
            {
                for (i=0; i<10; i++)
                    output = output + " " + message[i];
                _pvmTest.handleInput( output );
            }
	  break;
        }
 
        case _DoubleMsg:
        {
            int i;
            double message[] = new double[10]; // **2 changes
            String output = new String("Received double: "); // **1 superfic change

            System.out.println("ExamplePVM:  Got double"); // **1 superfic change

            if ( (stat = JavaPVM.upkdouble(message, 10, 1)) == 0) // **1 function change
            {
                for (i=0; i<10; i++)
                    output = output + " " + message[i];
                _pvmTest.handleInput( output );
            }

	  break;
        }
 
        case _FloatMsg:
        {
            int i;
            float message[] = new float[10]; // **2 changes 
            String output = new String("Received float: "); // **1 superfic change

            System.out.println("ExamplePVM:  Got float"); // **1 superfic change

            if ( (stat = JavaPVM.upkfloat(message, 10, 1)) == 0) // **1 function change
            {
                for (i=0; i<10; i++)
                    output = output + " " + message[i];
                _pvmTest.handleInput( output );
            }

          break;

        }
        case _LongMsg:
        {
            int i;
            long message[] = new long[10]; // **2 changes
            String output = new String("Received long: "); // **1 superfic change

            System.out.println("ExamplePVM:  Got long"); // **1 superfic change

            if ( (stat = JavaPVM.upklong(message, 10, 1)) == 0) // **1 function change
            {
                for (i=0; i<10; i++)
                    output = output + " " + message[i];
                _pvmTest.handleInput( output );
            }

          break;
        }

        case _ShortMsg:
        {
            int i;
            short message[] = new short[10]; // **2 changes 
            String output = new String("Received short: "); // **1 superfic change

            System.out.println("ExamplePVM:  Got short"); // **1 superfic change

            if ( (stat = JavaPVM.upkshort(message, 10, 1)) == 0) // **1 function change
            {
                for (i=0; i<10; i++)
                    output = output + " " + message[i];
                _pvmTest.handleInput( output );
            }

	  break;
        }

	case _TidMsg:
	{
	  int		rTid[] = new int[1];

	  if ( (stat = JavaPVM.upkint(rTid, 1, 1)) == 0 )	// 0 = success
	  {
	    System.out.println("ExamplePVM:  Received TID message: " + 
					rTid[0]);
	    _otherTid = rTid[0];
	  }
	  
	  break;
	}

        case _StartBench:
        {

          int i, j;
          char chararray[] = new char[2000];

          // The other client will be sending... we're receiving
          _isBenchmarking = true;
          _pvmTest.handleInput("Receiving benchmark command");

          // initialize the variables
          for(i=0; i<2000; i++)
              chararray[i] = 0;
 
          // send message saying we're ready to start receiving
          _pvmTest.handleInput("   Send confirmation message");
          JavaPVM.initsend(JavaPVM.PvmDataRaw);
          JavaPVM.send(_otherTid, _ConfirmBench);

          // begin the benchmarking
          // receive 100 character arrays from the other client
          _pvmTest.handleInput("   Waiting for the benchmark characters");
          i=0;
          while (i++ < 100)
          {
              JavaPVM.recv(_otherTid, _BenchChars);
              // unload the characters
              JavaPVM.upkstr(chararray);
          }

          // send confirmation that we received all the characters
          _pvmTest.handleInput("   Characters received.");
          JavaPVM.initsend(JavaPVM.PvmDataRaw);
          JavaPVM.send(_otherTid, _EndBench);

          // we're done-  set hold variable to false
          _isBenchmarking = false;

          _pvmTest.handleInput("   Bench completed");

	  break;
        }

        case _ConfirmBench:
        {
          // The other client has sent confirmation... we must have
          // sent a start bench to him... lets bench mark, then
 
          long starttime, endtime;
          int i=0;
          char chararray[] = new char[1000];
          String sendstring;

          // initialize variables
          _pvmTest.handleInput("Benchmarking: initializing vairables");
          for (i=0; i<1000; i++)
              chararray[i] = 70;  // just some value
          sendstring = new String(chararray);

          // block execution of everything else
          _isBenchmarking = true;

          //begin the benchmarking
          _pvmTest.handleInput("Benchmarking: start time set");
          starttime = System.currentTimeMillis();

          //send character arrays to the server
          i=0;
          while (i++<100)
          {
              // send characters
              JavaPVM.initsend(JavaPVM.PvmDataRaw);
              JavaPVM.pkstr(sendstring);
              JavaPVM.send(_otherTid, _BenchChars);
          }

          // wait for the endbench response
          _pvmTest.handleInput("Benchmarking: characters sent.. waiting for confirmation");
          JavaPVM.recv(_otherTid, _EndBench);

          // get end time and report
          endtime = System.currentTimeMillis();
          _pvmTest.handleInput(new String("Benchmarking complete."));
          _pvmTest.handleInput("   TransferTime = " + (endtime - starttime) + " milliseconds." );

          // remove the benchmarking hold
          _isBenchmarking = false;

          break;
        }


      }
    }
  }

	/*
	 * Used to clean things up in the event of a mishap.
	 */

  public int abort()
  {
    JavaPVM.exit();			// leave PVM process group

    System.exit(0);		// kill this whole application

    return(0);			// has to have return value
  }

	/*
	 *  Begin the benchmark process
	 */

  public void Benchmark()
  {
      // send the start bench code-  we'll wait for the confirm bench
      // before setting the isBenchmarking to true

     System.out.println("Start bench message sent");
     _pvmTest.handleInput("Dispached start bench message");
     JavaPVM.initsend(JavaPVM.PvmDataRaw);
     JavaPVM.send(_otherTid, _StartBench);
  }

  public boolean isBenchmarking()
  {
    return _isBenchmarking;
  }

	/*
	 * Sends a string to the other process
	 */

  public void sendMsg(String msg, String contents)
  {

    int i;

    if (_isBenchmarking) {

        System.out.println("**** In the middle of benchmarking... no message sent.");

    } else if (msg == "Send String") {

        System.out.println("Sending String to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pkstr(contents);
        JavaPVM.send(_otherTid, _StringMsg);
        System.out.println("Strings sent!");
    
    } else if (msg == "Send Integers") {

        int sendstring[] = new int[10];

        for (i=0; i<10; i++)
        {
            sendstring[i] = i;
        }

        System.out.println("Sending Integer array to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pkint(sendstring,10,1);
        JavaPVM.send(_otherTid, _IntMsg);
    
    } else if (msg == "Send Bytes") {

        byte sendstring[] = new byte[10];

        for (i=0; i<10; i+=2)
        {
            sendstring[i] = 0;
            sendstring[i+1] = 1;
        }

        System.out.println("Sending Byte array to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pkbyte(sendstring,10,1);
        JavaPVM.send(_otherTid, _ByteMsg);

    } else if (msg == "Send Doubles") {

        double sendstring[] = new double[10];

        for (i=0; i<10; i++)
        {
            sendstring[i] = 0.05 + (double) i;
        }

        System.out.println("Sending Double array to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pkdouble(sendstring,10,1);
        JavaPVM.send(_otherTid, _DoubleMsg);
    } else if (msg == "Send Floats") {

        float sendstring[] = new float[10];

        for (i=0; i<10; i++)
        {
            sendstring[i] = 0.05f + (float) i;
        }

        System.out.println("Sending Float array to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pkfloat(sendstring,10,1);
        JavaPVM.send(_otherTid, _FloatMsg);
    } else if (msg == "Send Long") {

        long sendstring[] = new long[10];

        for (i=0; i<10; i++)
        {
            sendstring[i] = (long) i;
        }

        System.out.println("Sending long array to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pklong(sendstring,10,1);
        JavaPVM.send(_otherTid, _LongMsg);
    } else if (msg == "Send Short") {

        short sendstring[] = new short[10];

        for (i=0; i<10; i++)
        {
            sendstring[i] = (short) i;
        }

        System.out.println("Sending short array to other process");
        JavaPVM.initsend(JavaPVM.PvmDataRaw);
        JavaPVM.pkshort(sendstring,10,1);
        JavaPVM.send(_otherTid, _ShortMsg);
    }
  }

}
