
#include <stdio.h>
#include <stdlib.h>
#include <pvm3.h>

		// Define message tag names as 
		// usable in switch statement

typedef enum 
{ 
  StringMsg = 1,
  ByteMsg   = 2,
  DoubleMsg = 3,
  FloatMsg  = 4,
  LongMsg   = 5,
  ShortMsg  = 6,
  IntMsg    = 7,
  TaskExit  = 8,
  TidMsg    = 9,
  TimeCode  = 20
} PvmMsgTag;

		// Function prototypes

void abort();
void echoMsg(int toTID, int msgTag, int bufId);

	/**
	 * Here's the program
	 */

void main ()
{
  static char pvmGroup[] = { "ExampleGroup" };
  
  int mytid = 0;
  int stat = 0;
  int bufId = 0;
  int bufInfo = 0;
  int msgtag = 0;
  int bytes = 0;
  int tid = 0;
    int sendTID[1];
  int rTid[1];


  mytid = pvm_mytid();

  stat = pvm_joingroup(pvmGroup);

  if (stat < 0)
  {
    printf("Couldn't join ExamplePVM group: stat = %d\n", stat);
    abort();
  }

		// then wait for PvmTest to join group

  printf("pvmEchoTest:  Waiting for other processes.\n");

  stat = pvm_barrier(pvmGroup, 2);

  if (stat < 0)  // oops
  {
    printf("barrier failed: stat = %d Exiting...\n", stat);
    abort();
  }

  printf("pvmEchoTest:  Got one.\n");

		// then send my TID out to other process

  sendTID[0] = mytid;

  pvm_initsend(PvmDataRaw);
  pvm_pkint(sendTID, 1, 1);
  pvm_bcast(pvmGroup, TidMsg);

		// infinite loop waiting for and handling messages

  printf("pvmEchoTest:  Starting up PVM watcher...\n");

  while (1)
  {
	// blocking receiving for any message

    bufId = pvm_recv(-1, -1);

    printf("Got something!\n");

		// ignore errors

    if (bufId <= 0)
    {
      printf("bufId = %d\n", bufId);
      continue;
    }

		// get bufinfo to determine message type

    bufInfo = pvm_bufinfo(bufId, &bytes, &msgtag, &tid);

		// switch on msgtag value

    switch ((PvmMsgTag)msgtag)
    {
      case (TaskExit):
	printf ("Exit command received via pvm.\n");
	abort();
	break;

      case (StringMsg):
	printf("pvmEchoTest:  Received string ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (IntMsg):
	printf("pvmEchoTest: Received ints ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (ByteMsg):
	printf("pvmEchoTest:  Received bytes ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (DoubleMsg):
	printf("pvmEchoTest:  Received doubles ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (FloatMsg):
	printf("pvmEchoTest:  Received floats ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (LongMsg):
	printf("pvmEchoTest:  Received longs ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (ShortMsg):
	printf("pvmEchoTest:  Received short ... echoing\n");
	echoMsg (tid, msgtag, bufId);
	break;

      case (TidMsg):
	stat = pvm_upkint(rTid, 1, 1);
	if (stat == 0)	// 0 = success
	{
	  printf("pvmEchoTest:  Received TID message: %d\n", rTid[0]);
	  // Don't really care about this, we echo to anybody!
	}
	break;

      default:
	printf("pvmEchoTest: Bad message tag received = %d\n", msgtag);
	break;
    }
      
  }
}


	/*
	 * Used to clean things up in the event of a mishap.
	 */

void abort()
{
  pvm_exit();			// leave PVM process group

  exit(0);			// kill this whole application
}

	/*
	 * Echos the last buffer received to the other process
	 */

void echoMsg(int toTID, int msgTag, int bufId)
{
  pvm_setsbuf (bufId);
  pvm_send(toTID, msgTag);
  pvm_freebuf (bufId);
}
