// ####################################################################
// jPVM - an interface to the Parallel Virtual Machine for Java
// ####################################################################
//
// --------------------------------------------------------------------
// $Id: jPVM.java,v 1.18 1998/04/10 17:43:50 dave Exp $
// --------------------------------------------------------------------
//
// Copyright (c) 1996 by:
//
//		David A.Thurman
//		Center for Human-Machine Systems Research
//		School of Industrial and Systems Engineering
//		Georgia Institute of Technology
//		Atlanta, Georgia, USA
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// --------------------------------------------------------------------
//
// $Log: jPVM.java,v $
// Revision 1.18  1998/04/10 17:43:50  dave
// name change to jPVM
// added Javadoc documentation
//
// Revision 1.17  1998/01/19 16:29:08  dave
// Modified library name to include release information.
//
// Revision 1.16  1998/01/19 16:03:26  dave
// Updated distribution id.
// mv Makefile t.Makefile
//
// Revision 1.15  1997/11/12 19:58:57  dave
// Changes associated with methods that were removed in pvm3.4beta:
// pvm_delete
// pvm_insert
// pvm_lookup
// pvm_setmwid
// pvm_getmwid
//
// Revision 1.14  1997/09/20 00:05:24  dave
// Added trecv
//
// Revision 1.13  1997/09/17 11:41:22  dave
// Removed pvm_ prefix on all methods, since they will now be
// referenced as jPVM.method()
//
// Revision 1.12  1997/09/10 12:33:02  dave
// Removed methods which will never be implemented (packf, upackf, etc.).
// Removed methods involving unsigned variables (upku* and pku*).
// Added method definition of pkboolean and upkboolean.
// Changed indentation of commented out methods.
//
// Revision 1.11  1997/08/21 15:18:34  dave
// Changed names of routines back to having the '' prefix.
// Decided too many people would have to change their code.  Besides,
// I like the fact that the routines have the same names in PVM
// and jPVM.
// -Added some minor output information in the static load of the
// library so you know that it is really happening.
// Changed constructor to simply call mytid() to enroll the process.
// Removed setDebug() method.  unused.
//
// Revision 1.10  1997/08/21 14:06:07  dave
// Changed constant names back to PVM standard with leading 'Pvm'
//
// Revision 1.9  1997/08/20 15:54:26  dave
// Removed Pvm_ prefixes on all constants.
// Removed  prefixes on all methods.
//
// Revision 1.8  1996/12/16 21:03:16  dave
// Changed upkstr to fix problem related to overwritting Strings
// cached by Java.  Method now uses char[] argument instead of String.
// Added setDebug() routine to replace compile time flag.
//
// Revision 1.7  1996/06/07 10:34:23  dave
// Added Gnu copyright/license notice
//
// Revision 1.6  1996/05/31 10:30:37  dave
// Renamed constants to include 'Pvm' in keeping with C version
//
// Revision 1.5  1996/05/30 14:42:55  dave
// Renamed object class from PVM to jPVM.
// Added tasks and hosts to class.
//
// Revision 1.4  1996/05/30 14:18:32  dave
// Removed run() method.  PVM class is now an abstract class
// that must be subclassed for use.
//
// Revision 1.3  1996/05/23 14:16:36  dave
// Added reg_hoster and reg_tasker
//
// Revision 1.2  1996/05/23 14:03:53  dave
// Changed precv, psend, and upkbyte to use byte[] arrays for their
// buffers instead of Java Strings of char[] arrays.
//
// Revision 1.1  1996/05/21 13:45:00  dave
// Initial revision
//
//
// --------------------------------------------------------------------

	/**
	 * wrapper class for PVM C library
	 */

public abstract class jPVM 
{
	/* 
	 * define the constants used by PVM ---
	 *
	 * These are merely copied and modified from pvm3.h so
	 * if you don't know what they are for, check your PVM
	 * installation documents.  
	 */

	// Data packing styles for initsend

  public final static int PvmDataDefault = 0;
  public final static int PvmDataRaw = 1;
  public final static int PvmDataInPlace = 2;
  public final static int PvmDataFoo = 3;

	// spawn options

  public final static int PvmTaskDefault = 0;
  public final static int PvmTaskHost = 1;
  public final static int PvmTaskArch = 2;
  public final static int PvmTaskDebug = 4;
  public final static int PvmTaskTrace = 8;
  public final static int PvmMppFront = 16;
  public final static int PvmHostCompl = 32;

	// notify types

  public final static int PvmTaskExit = 1;
  public final static int PvmHostDelete = 2;
  public final static int PvmHostAdd = 3;

	// for setopt and getopt

  public final static int PvmRoute = 1;
  public final static int PvmDontRoute = 1;
  public final static int PvmAllowDirect = 2;
  public final static int PvmRouteDirect = 3;
  public final static int PvmDebugMask = 2;
  public final static int PvmAutoErr = 3;
  public final static int PvmOutputTid = 4;
  public final static int PvmOutputCode = 5;
  public final static int PvmTraceTid = 6;
  public final static int PvmTraceCode = 7;
  public final static int PvmFragSize = 8;
  public final static int PvmResvTids = 9;
  public final static int PvmSelfOutputTid = 10;
  public final static int PvmSelfOutputCode = 11;
  public final static int PvmSelfTraceTid = 12;
  public final static int PvmSelfTraceCode = 13;
  public final static int PvmShowTids = 14;
  public final static int PvmPollType = 15;
  public final static int PvmPollConstant = 1;
  public final static int PvmPollSleep = 2;
  public final static int PvmPollTime = 16;

	// for settmask and gettmask

  public final static int PvmTaskSelf = 0;
  public final static int PvmPvmTaskChild = 1;

	// libpvm error codes

  public final static int PvmOk = 0;
  public final static int PvmBadParam = -2;
  public final static int PvmMismatch = -3;
  public final static int PvmOverflow = -4;
  public final static int PvmNoData = -5;
  public final static int PvmNoHost = -6;
  public final static int PvmNoFile = -7;
  public final static int PvmNoMem = -10;
  public final static int PvmBadMsg = -12;
  public final static int PvmSysErr = -14;
  public final static int PvmNoBuf = -15;
  public final static int PvmNoSuchBuf = -16;
  public final static int PvmNullGroup = -17;
  public final static int PvmDupGroup = -18;
  public final static int PvmNoGroup = -19;
  public final static int PvmNotInGroup = -20;
  public final static int PvmNoInst = -21;
  public final static int PvmHostFail = -22;
  public final static int PvmNoParent = -23;
  public final static int PvmNotImpl = -24;
  public final static int PvmDSysErr = -25;
  public final static int PvmBadVersion = -26;
  public final static int PvmOutOfRes = -27;
  public final static int PvmDupHost = -28;
  public final static int PvmCantStart = -29;
  public final static int PvmAlready = -30;
  public final static int PvmNoTask = -31;
  public final static int PvmNoEntry = -32;
  public final static int PvmDupEntry = -33;

	// data types for reduce(), send, and precv()

  public final static int PVM_STR = 0;
  public final static int PVM_BYTE = 1;
  public final static int PVM_SHORT = 2;
  public final static int PVM_INT = 3;
  public final static int PVM_FLOAT = 4;
  public final static int PVM_CPLX = 5;
  public final static int PVM_DOUBLE = 6;
  public final static int PVM_DCPLX = 7;
  public final static int PVM_LONG = 8;
  public final static int PVM_USHORT = 9;
  public final static int PVM_UINT = 10;
  public final static int PVM_ULONG = 11;

        /**
         * Adds the computers named in <EM>hosts</EM> to the configuration
         * of computers making up the virtual machine.  The names should
        * have the same syntax as lines of a <STRONG>pvmd</STRONG>
         * hostfile (see man page for pvmd3):  A host name followed by
         * options of the form xx=y.
         *
         * @param hosts An array of Strings naming the hosts to be
         * added.  Pvmd must already be installed and the user must have
        * an account on the specified hosts.
         *
         * @param infos An integer array of length equal to <EM>hosts</EM>
         * which returns the status for each host.  Values less than
         * zero indicate an error, while positive values are TIDS of the
        * new hosts.
         *
         * @return status code.
         * If addhosts() is successfull, <EM>info</EM> will be equal to
         * <EM>hosts.length</EM>.  Partial success if indicated by 0 &lt;
         * info &lt; hosts.length and total failure by info &lt; 1. The array
         * <EM>infos</EM> can be checked to determine which host caused the
         * error.
         *
         * @see jPVM#mstat
         * @see jPVM#config
         */

  public static native int addhosts(String hosts[], int infos[]);
        
	/**
         * Blocks the calling process until <EM>count</EM> members of
         * the <EM>group</EM> have called barrier().  The count argument
         * is required because processes could be joining the given
         * group after other processes have called barrier().  Thus,
         * PVM doesn't know how many group member to wait for at any
         * given instant.  Although <EM>count</EM> can be set less, it
         * is typically the total number of members of the group.  So,
         * the logical function of the barrier() call is to provide a
         * group synchronization.  During any given barrier call, all
         * participating group members must call barrier() with the same
         * <EM>count</EM> value.  Once a given barrier has been
         * successfully passed, barrier() may be called again by the same
         * group using the same group name.
         *
         * @param group Character string group name.  The group must
         * exist and the calling process must be a member of the group.
         *
         * @param count Integer specifying the number of group members
         * that must call barrier() before they are all released.  Though
         * not required, count is expected to be the total number of
         * members of the specified group.
         *
         * @return status code.
         * If barrier() is successful, <EM>info</EM> will be 0.  If some
         * error occurs, then <EM>info</EM> will be &lt; 0.
         *
         * @see jPVM#joingroup
         */

  public static native int barrier(String group, int count);

        /**
         * Broadcasts a message stored in the active send buffer to all
         * members of <EM>group</EM>.  The broadcast messages is not sent
         * back to the sender.  Any PVM task can call bcast(), it need not
         * be a member of the group.  The content of the message can be
         * distinguished by <EM>msgtag</EM>.
         * <P>
         * bcast() is asynchronoous.  Computation on the sending processor
         * resumes as soon as the message is safely on its way to the
         * receiving processors.  This is in contrast to synchronous
         * communication, during which computation on the sending
         * processor halts until a matching receive is executed by all the
         * receiving processors.
         * <P>
         * bcast() first determines the tids of the group members by
         * checking the group data base.  A multicast is performed to
         * these tids.  If the group is changed during a broadcast, the
         * change will not be reflected in the broadcast.  Multicasting is
         * not supported by most multiprocessor vendors.  Typically, their
         * native calls only support broadcasting to <EM>all</EM> the
         * user's processes on a multiprocessor.  Because of this
         * omission, bcast() may not be an efficient communication method
         * on some multiprocessors.
         *
         * @param group Character string group name of an existing group
         *
         * @param msgtag Integer message tag supplied by user.  msgage tag
         * should be &gt;=0.  It allows the user's program to distinguish
         * between different kinds of messages.
         *
         * @return status code.  Values less than zero indicate an
         * error.
         *
         * @see jPVM#joingroup
         */

  public static native int bcast(String group, int msgtag);

        /**
         * Returns information about the requested message buffer.
         * Typically it is used to determing facts about the last received
         * messages such as its size of source.  bufinfo() is especially
         * useful when an application is able to receive an incoming
         * message, and the action taken depends on the source
         * <EM>tid</EM> and the <EM>msgtag</EM> associated with the
         * message that comes in first.
         * <P>
         * <STRONG>jPVM note:</STRONG>
         * Note that the parameters <EM>bytes, msgtag, and tid</EM> are
         * all passed as Integers.  Upon succesful completion, use their
         * intValue() method to get the actual values for those
         * parameters.
         *
         * @param bufid int specifying a particular message buffer
         * identifier.
         *
         * @param bytes Integer return the length in byts of the entire
         * message.
         *
         * @param msgtag Integer returning the message label.  Useful
         * when the message was received with a wildcard msgtag (-1).
         *
         * @param tid Integer returning the source of the message.
         * Useful when the message was received with a wildcard tid
         * (-1).
         *
         * @return Status code:
         * Values less than zero indicate an error.
         */
  
  public static native int bufinfo(int bufid, Integer bytes, 
				Integer msgtag, Integer tid);


//  public static native int catchout(File *ff);
//  public static native int config(Integer nhost, Integer narch, 
//				PvmHostInfo phi[]);

        /**
         * Deletes entry &lt;name, index&gt; from the pvmd database.
         *
         * @param name The class name, a null-terminated string
         *
         * @param index The class index, &gt;=0.
         *
         * @return Status code:  If successful, delete() returns
         * PvmOk, otherwise it returns a negative error code.
         *
         * @see jPVM#insert
         * @see jPVM#lookup
         */

  public static native int delete(String name, int index);

        /**
         * Deletes the computers named in <EM>hosts</EM> from the
         * existing virtual machine configuration.  All PVM processses and
         * the pvmd running on these computers are killed as the computer
         * is deleted.
         *
         * If a host failes, PVM will continue to function and will
         * automatically delete the host from the virtual machine.  An
         * application can be notified of a host failure through notify().
         * It is still the responsibility of the application developer to
         * make an application tolerant of host failure.
         *
         * @param hosts An array of Strings containing the names of the
         * machines to be deleted.
         *
         * @param infos int array of same length as <EM>hosts</EM> which
         * contains the status code returned by the router for the
         * individual hosts.  Values less than zero in this array
         * indicate an error.
         *
         * @return Status code:
         * If delhosts() is successful, return value <EM>info</EM> will equal
         * <EM>hosts.length</EM>.  Partial success is indicated by
         * 1&lt;=info&lt;hosts.length, and total failure by info&lt1.  The
         * array <EM>infos</EM> can be checked to determine which host
         * caused the error.
         *
         * @see jPVM#addhosts
         * @see jPVM#notify
         */

  public static native int delhosts(String hosts[], int infos[]);

        /**
         * Tells the local pvmd that this process is leaving PVM.  This
         * routine does not kill the process, which can continue to
         * perform tasks just like any other serial process.
         * <P>
         * jPVM.exit() should be called by all PVM processes before they
         * stop or exit for good.  It <STRONG>must</STRONG> be called by
         * processes that were not started with jPVM.spawn().
         *
         * @return Status code:
         * Values less than zero indicate an error.
         */

  public static native int exit();

	/**
	 * Frees the memory associated with the specified message
	 * buffer, <EM>bufid</EM>.  Message buffers are created using
	 * <CODE>mkbuf(), initsend(), and recv()</CODE>.  See PVM
	 * version for full discussion of buffer use.
	 *
	 * @return Status code:  Values less than 0 indicate an erro.
	 *
	 * @see jPVM#mkbuf
	 * @see jPVM#initsend
	 * @see jPVM#recv
	 */

  public static native int freebuf(int bufid);

//  public static native int gather(void *result, void *data, int count, 
//			int datatype, int msgtag, String group, int rootginst);


	/**
	 * Returns the unique instance number corresponding to the task
	 * identified by <EM>tid</EM> in the group identified by
	 * <EM>group</EM>.
	 *
	 * @return Status code:  Values less than zero indicate an error.
	 *
	 * @see jPVM#joingroup
	 * @see jPVM#gettid
	 */

  public static native int getinst(String group, int tid);

	/**
	 * Returns value of libpvm options.  <EM>what</EM> defines which
	 * option value to return.  See PVM description for full
	 * details.
	 *
         * @see jPVM#setopt
	 */

  public static native int getopt(int what);

	/**
	 * Returns the message buffer identifier <EM>bufid</EM> for the
	 * active receive buffer, or 0 if there is no current buffer.
	 *
	 * @see jPVM#getsbuf
	 */

  public static native int getrbuf();

	/**
	 * Returns the message buffer identifier <EM>bufid</EM> for the
	 * active send buffer, or 0 if there is no current buffer.
	 *
	 * @see jPVM#getrbuf
	 */

  public static native int getsbuf();

	/**
	 * Returns the tid of the PVM process identified by the group
	 * name <EM>group</EM> and the instance number <EM>inum</EM>.  
	 * 
	 * @return Status code: 0 if succesful, less than zero for errors.
	 *
	 * @see jPVM#joingroup
	 * @see jPVM#getinst
	 */

  public static native int gettid(String group, int inum);

//  public static native int gettmask(int who, Pvmtmask mask);

	/**
	 *k
	 */

  public static native int gsize(String group);

	/**
	 *
	 */

  public static native int halt();

//  public static native int hostsync(int host, struct timeval *clk, 
//				struct timeval *delta);

	/**
	 *
	 */

  public static native int initsend(int format);

	/**
	 *
	 */

  public static native int joingroup(String group);

	/**
	 *
	 */

  public static native int kill(int tid);

	/**
	 *
	 */

  public static native int lvgroup(String group);

	/**
	 *
	 */

  public static native int mcast(int tids[], int msgtag);

	/**
	 *
	 */

  public static native int mkbuf(int encoding);

	/**
	 *
	 */

  public static native int mstat(String host);

	/**
	 *
	 */

  public static native int mytid();

	/**
	 *
	 */

  public static native int notify(int what, int msgtag, int tids[]);

	/**
	 *
	 */

  public static native int nrecv(int tid, int msgtag);

	/**
	 *
	 */

  public static native int parent();
	
	/**
	 *
	 */

  public static native int pkboolean(boolean xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkbyte(byte xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkcplx(float cp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkdcplx(double zp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkdouble(double dp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkfloat(float fp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkint(int ip[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pklong(long ip[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkshort(short jp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int pkstr(String sp);

	/**
	 *
	 */

  public static native int perror(String msg);

	/**
	 *
	 */

  public static native int precv(int tid, int msgtag, byte buf[], int len,
		int datatype, Integer atid, Integer atag, Integer alen);

	/**
	 *
	 */

  public static native int probe(int tid, int msgtag);

	/**
	 *
	 */

  public static native int psend(int tid, int msgtag, byte buf[], 
				int len, int datatype);

	/**
	 *
	 */

  public static native int pstat(int tid);

	/**
	 *
	 */

  public static native int recv(int tid, int msgtag);

	/**
	 *
	 */

  public static native int reg_hoster();

//  public static native int reg_rm(PvmHostInfo phi);

	/**
	 *
	 */

  public static native int reg_tasker();

//  public static native int scatter(void *result, void *data, int count, 
//		int datatype, int msgtag, String group, int rootginst);

	/**
	 *
	 */

  public static native int send(int tid, int msgtag);

	/**
	 *
	 */

  public static native int sendsig(int tid, int signum);

	/**
	 *
	 */

//  public static native int serror();

	/**
	 *
	 */

  public static native int setopt(int what, int val);

	/**
	 *
	 */

  public static native int setrbuf(int bufid);

	/**
	 *
	 */

  public static native int setsbuf(int bufid);

	/**
	 *
	 */

//  public static native int settmask(int who, Pvmtmask mask); 

	/**
	 *
	 */

  public static native int spawn(String task, String argv[], int flag, 
				String where, int tids[]);

	/**
	 *
	 */

  public static native int start_pvmd(int argc, String argv[], int block);

	/**
	 *
	 */

//  public static native int tasks(int where, Integer ntask, PvmTaskInfo pti);

	/**
	 *
	 */

  public static native int tidtohost(int tid);

	/**
	 *
	 */

  public static native int trecv(int tid, int msgtag, int sec, int usec);

	/**
	 *
	 */

  public static native int upkboolean(boolean xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkbyte(byte xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkcplx(float xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkdcplx(double xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkdouble(double xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkfloat(float xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkint(int xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upklong(long xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkshort(short xp[], int nitem, int stride);

	/**
	 *
	 */

  public static native int upkstr(char sp[]);
  
        /**
         * Allocates an internal character array of <EM>size</EM> characters,
         * calls native method jPVM. upkstr(char[]), and converts
         * the resulting contents of the character array to a String
         * with excess white-space trimmed off.
         * <P>
         * If you can live with a default size of 256 characters in
         * the character array, you can simply call jPVM.upkstr().
         * <P>
         * If you want complete control over the character array
         * allocation (e.g., you want to use one array over and over
         * for String unpacking, use jPVM.upkstr(char[]).
         *
         * @param size - Size of character array to use for unpacking
         * from PVM buffer.  Should be greater than or equal to length
         * of maximum expected string (including null terminator).
         *
         * @return String allocated from character array with excess
         * trailing white space trimmed off.
         *
         * @see jPVM#upkstr(char[])
         * @see jPVM#upkstr()
         */
 
  public static String upkstr(int size)
  {
    char charStr[] = new char[size];
 
    int info = upkstr(charStr);
 
    if (info >= 0)
    {
      String retStr = new String(charStr).trim();
      return retStr;
    }
    else
    {
      System.err.println(
        "Can't unpack string in jPVM.upkstr() -- info = " + info);
      return null;
    }
  }
        /**
         * Same as jPVM.upkstr(int), but with default character
         * array size of 256 characters (sufficient for most
         * applications).
         * <P>
         * If you want greater control over the amount
         * of memory allocated for Strings (e.g., you need larger
         * Strings, or know you'll never need Strings that large,
         * use jPVM.upkstr(int).
         * <P>
         * If you want complete control over the character array
         * allocation (e.g., you want to use one array over and over
         * for String unpacking), use jPVM.upkstr(char[]).
         *
         * @return String allocated from character array with excess
         * trailing white space trimmed off.
         *
         * @see jPVM#upkstr(char[])
         * @see jPVM#upkstr(int)
         */
 
  public static String upkstr()
  {
    return upkstr(256);
  }

	/**
	 *
	 */

  static 
  {
    System.out.print("Loading jPVM-v1.1.4 shared library...");

    System.loadLibrary("jPVM-v1.1.4");

    System.out.println("done.");
  }

}	// end class jPVM
